#version 150

uniform sampler2D DiffuseSampler;
uniform sampler2D DiffuseDepthSampler;
uniform sampler2D TranslucentSampler;
uniform sampler2D TranslucentDepthSampler;
uniform sampler2D ItemEntitySampler;
uniform sampler2D ItemEntityDepthSampler;
uniform sampler2D ParticlesSampler;
uniform sampler2D ParticlesDepthSampler;
uniform sampler2D WeatherSampler;
uniform sampler2D WeatherDepthSampler;
uniform sampler2D CloudsSampler;
uniform sampler2D CloudsDepthSampler;

in vec2 texCoord;

#define NUM_LAYERS 6
#define NUM_RECURSIONS 7

#define MARKING_COLOR vec4(1.0, 0.2, 0.4, 1.0)

vec4 color_layers[NUM_LAYERS];
float depth_layers[NUM_LAYERS];
int active_layers = 0;

out vec4 fragColor;

void try_insert(vec4 color, float depth) {
    if (color.a == 0.0) {
        return;
    }

    color_layers[active_layers] = color;
    depth_layers[active_layers] = depth;

    int jj = active_layers++;
    int ii = jj - 1;
    while (jj > 0 && depth_layers[jj] > depth_layers[ii]) {
        float depthTemp = depth_layers[ii];
        depth_layers[ii] = depth_layers[jj];
        depth_layers[jj] = depthTemp;

        vec4 colorTemp = color_layers[ii];
        color_layers[ii] = color_layers[jj];
        color_layers[jj] = colorTemp;

        jj = ii--;
    }
}

vec3 blend(vec3 dst, vec4 src) {
    return (dst * (1.0 - src.a)) + src.rgb;
}

void main() {
    vec2 offsetPos = texCoord;
    vec4 translucentCol = texture(TranslucentSampler, offsetPos);
    int r;
    vec4 recursionMap;

    for (r = 0; r < NUM_RECURSIONS; r++) {
        recursionMap = texture(TranslucentSampler, offsetPos);

        if (recursionMap.a >= 0.9 && recursionMap.b < 0.1) {
            offsetPos.x = recursionMap.r;
            offsetPos.y = recursionMap.g;
        }
        else {
            break;
        }
    }

    color_layers[0] = vec4(texture(DiffuseSampler, offsetPos).rgb, 1.0);
    depth_layers[0] = texture(DiffuseDepthSampler, offsetPos).r;
    active_layers = 1;

    vec4 newCol = vec4(0.0, 0.0, 0.0, 0.0);
    if (recursionMap.a > 0.1 && recursionMap.b >= 0.1) {

        if (recursionMap.b >= 0.1 && recursionMap.b < 0.2 && r == 1) {
            newCol = MARKING_COLOR;
        }
        else if (recursionMap.b >= 0.2 && recursionMap.b < 0.3 && r == 2) {
            newCol = MARKING_COLOR;
        }
        else if (recursionMap.b >= 0.3 && recursionMap.b < 0.4 && r == 3) {
            newCol = MARKING_COLOR;
        }
        else if (recursionMap.b >= 0.4 && recursionMap.a >= 0.6) {
            newCol = MARKING_COLOR;
        }
    }

    try_insert(newCol, texture(TranslucentDepthSampler, offsetPos).r);
    try_insert(texture(ItemEntitySampler, offsetPos), texture(ItemEntityDepthSampler, offsetPos).r);
    try_insert(texture(ParticlesSampler, offsetPos), texture(ParticlesDepthSampler, offsetPos).r);
    try_insert(texture(WeatherSampler, offsetPos), texture(WeatherDepthSampler, offsetPos).r);
    try_insert(texture(CloudsSampler, offsetPos), texture(CloudsDepthSampler, offsetPos).r);

    vec3 texelAccum = color_layers[0].rgb;
    for (int ii = 1; ii < active_layers; ++ii) {
        texelAccum = blend(texelAccum, color_layers[ii]);
    }

    fragColor = vec4(texelAccum.rgb, 1.0);
}
